/******************************************************************************/
/*                                                                            */
/* DESCRIPTION DE LA BIBLIOTHEQUE TRACE                                       */
/*                                                                            */
/******************************************************************************/
/* Auteur               : Ph. RUELEN                                          */
/* Date                 : 27/04/95                                            */
/*                                                                            */
/*    27.04.95 : Creation de la bibliotheque                                  */
/*                                                                            */
/******************************************************************************/
/*           LISTE DES FONCTIONS ET PROCEDURES DE LA BIBLIOTHEQUE             */
/******************************************************************************/
/*                                                                            */
/*                           TRACE.C                                          */
/*                           -------                                          */
/*                                                                            */
/*               EcrireEnteteTrace()                                          */
/*               FermerFichierTrace()                                         */
/*               FormaterLeFichierTrace()                                     */
/*               LireNiveaux()                                                */
/*               InitTrace()                                                  */
/*               LireEnteteTrace()                                            */
/*               MajEnteteTrace()                                             */
/*               OuvreFichierTrace()                                          */
/*               PositionnerDansFichierTrace()                                */
/*               Trace()                                                      */
/*                                                                            */
/******************************************************************************/

/*== Includes systemes necessaires a la bibliotheque =========================*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <time.h>


/*== Includes non-systemes necessaires a la bibliotheque =====================*/

#include "bibtrace.h"


/*== Constantes internes a la bibliotheque ===================================*/

#define BIBTRACE       "BIBTRACE"


#define NIVEAU_TRACE_MIN  1                   /* niveau minimum autorise      */
#define NIVEAU_TRACE_MAX  16                  /* niveau maximum autorise      */

#define LECTURE                   "r"   /* lecture seule                      */
#define ECRITURE                  "w"   /* ecriture                           */
#define LECT_ECR                  "r+"  /* lecture ecriture                   */
#define CREAT_APPEND              "a"   /* creation ou ajout                  */

#define LG_PATH_FICHIER           80
#define LG_MAX_MSG_TRACE         600
#define TAILLE_ENTETE           2*40
#define LG_MSG_FIXE_TRACE         36
#define LG_NOM_PORT               10
#define LG_LIGNE                 256

#define NB_MAX_TRACE      2000     /* nombre maximum de trace autorise pour */
                                   /* la gestion des consignes */
#define TRACE_NORMAL        ' '    /* pas de consigne particuliere de trace */
#define TRACE_FORCE         'F'    /* consigne de forcage de trace */
#define TRACE_INHIBE        'I'    /* consigne d'inhibition de trace */
#define PAS_DE_CONSIGNE     'P'    /* pas d'utilisation des consignes */
#define CONSIGNE            'C'    /* utilisation des consignes */

struct S_ENTETE_TRACE {
       int NbMaxiTraces;       /* Nombre total maxi des traces dans le fichier*/
       int LgMsgVarTrace;      /* Longueur de la partie variable des messages */
       int NiveauAVisualiser;  /* Niveaux que l'on doit visualiser a l'ecran  */
       int NiveauAEnregistrer; /* Niveaux a stocker sur disque dans un fichier*/
       int NiveauAImprimer;    /* Niveaux que l'on doit imprimer              */
       int NiveauADeboguer;    /* Niveaux dont on donne le nom du fichier     */
       int Reserve;            /* Reserve pour les extensions a venir         */
       int pEcrTrace;          /* Numero de la prochaine trace a ecrire       */
};
typedef struct S_ENTETE_TRACE T_ENTETE_TRACE;


/*== Prototypes des fonctions importees pour la biliotheque ==================*/

PROCEDURE     EcrireEnteteTrace(void);
PROCEDURE     FermerFichierTrace(FILE * pFichier);
PROCEDURE     FormaterLeFichierTrace(void);
FONCTION  int LireEnteteTrace(void);
FONCTION  int LireNiveaux(void);
PROCEDURE     MajEnteteTrace(void);
FONCTION  int OuvreFichierTrace(char *NomFichier,
                                char *ModeOuverture,FILE **pFichier);
PROCEDURE     PositionnerDansFichierTrace(FILE *pFichier, long Offset, int Origine);

/*== Prototypes des fonctions et procedures internes a la biliotheque ========*/


/*== Variables globales a exporter de la bibliotheque ========================*/

char _Fich_[LG_FICHIER+1];       /* Nom du fichier emetteur (Trace ou Handler) */
int  _Ligne_;                    /* Ligne d'appel dans le fichier C            */


/*== Variables globales internes de la bibliotheque ==========================*/

static char MsgTrace[LG_MAX_MSG_TRACE];
static char MsgVarTrace[LG_MAX_MSG_TRACE];
static char PortImprimante[LG_NOM_PORT]="LPT1      ";
static char FichierTrace[LG_PATH_FICHIER];    /* Nom du fichier     */
static T_ENTETE_TRACE InfoTrace;              /* Entete fichier Trc */
static unsigned int   Masque[]={0x0000,0x0001,0x0002,0x0004,0x0008,
                                       0x0010,0x0020,0x0040,0x0080, /* 2 exp N*/
                                       0x0100,0x0200,0x0400,0x0800,
                                       0x1000,0x2000,0x4000,0x8000};
static char TabCons[NB_MAX_TRACE+1];
T_PTR_FONC  FoncNivHandler;   /* fonction a appeler sur une trace de niveau */
                              /* HANDLER */


/******************************************************************************/
/************** CORPS DES FONCTIONS INTERNES DE LA BIBLIOTHEQUE ***************/
/******************************************************************************/

/************* TRACE pour la mise au point de la bibliotheque ****************/ 
                                                                                
PROCEDURE TRACEINT(char * format, ...)                                             
debut
  va_list pArg;

  va_start(pArg,format);
  vprintf(format,pArg);
  va_end(pArg);
fin

/************** HANDLER pour la mise au point de la bibliotheque *************/

/*PROCEDURE HANDLER(char * format, ...)
debut
  va_list pArg;

  printf("HANDLER ! > ");
  va_start(pArg,format);
  vprintf(format,pArg);
  printf("\n\n");
  va_end(pArg);
fin*/


/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : EcrireEnteteTrace                            */
/******************************************************************************/
/* BUT : Cette procedure permet d'ecrire l'entete du fichier trace en cours   */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* FichierTrace  | | |x| char *     | fichier DOS | Nom du fichier trace      */
/* InfoTrace     | | |x| T_ENTETE_TRACE           | Contient l'entete du      */
/*               | | | |            |             | fichier trace             */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : EcrireEnteteTrace                             */
/* Ouvrir le fichier trace                                                    */
/* Recopier la strucuture InfoTrace en ASCII                                  */
/* Recopier les commentaires                                                  */
/* Fermer le fichier                                                          */
/******************************************************************************/
PROCEDURE EcrireEnteteTrace(void)
debut
   FILE *pFicTrace;    /* Descripteur de fichier                              */

   SI (OuvreFichierTrace(FichierTrace,LECT_ECR,&pFicTrace)==OK) ALORS
      SI (fprintf(pFicTrace,"%04.4d %03.3d %04.4x %04.4x %04.4x %04.4x %04.4d %04.4d",
                       InfoTrace.NbMaxiTraces,
                       InfoTrace.LgMsgVarTrace,
                       InfoTrace.NiveauAVisualiser,
                       InfoTrace.NiveauAEnregistrer,
                       InfoTrace.NiveauAImprimer,
                       InfoTrace.NiveauADeboguer,
                       InfoTrace.Reserve,
                       InfoTrace.pEcrTrace) < 0)  ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI
      SI (fprintf(pFicTrace,"\nNBTR LGM VISU ENRG IMPR DEBG RSRV PTEC\n") <0) ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI

      FermerFichierTrace(pFicTrace);
  FIN_SI
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : FermerFichierTrace                           */
/******************************************************************************/
/* BUT : Ferme un fichier; si cela se passe mal, on appelle l'handler.        */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/*               | | | |            |             |                           */
/* NomFichier    |X| | |char*       |             |Nom du fichier a fermer.   */
/* pFichier      |X| | |FILE*       |             |Handle de fichier.         */
/*               | | | |            |             |                           */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : FermerFichierTrace                            */
/*                                                                            */
/******************************************************************************/
PROCEDURE FermerFichierTrace(FILE * pFichier)
debut
  SI fclose(pFichier) !=0 ALORS
     Handler(BIBTRACE,ERR_FERME_FICH);
  FIN_SI
fin

/******************************************************************************/
/* PROCEDURE FormaterLeFichierTrace(void)                                     */
/******************************************************************************/
/* BUT : Cette procedure permet d'ecrire l'entete du fichier trace en cours   */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* TabCons       | | |x|  char *    |             | consignation de trace     */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
PROCEDURE FormaterLeFichierTrace(void)
debut
   FILE *pFicTrace;         /* Descripteur de fichier                         */
   char FormatVariable[20]; /* Format temporaire                              */
   int  iMessage;           /* Compteur de message vierge a enregistrer       */


   SI (OuvreFichierTrace(FichierTrace,ECRITURE,&pFicTrace)==OK) ALORS
      SI (fprintf(pFicTrace,"%04.4d %03.3d %04.4x %04.4x %04.4x %04.4x %04.4d %04.4d",
                       InfoTrace.NbMaxiTraces,
                       InfoTrace.LgMsgVarTrace,
                       InfoTrace.NiveauAVisualiser,
                       InfoTrace.NiveauAEnregistrer,
                       InfoTrace.NiveauAImprimer,
                       InfoTrace.NiveauADeboguer,
                       InfoTrace.Reserve,
                       InfoTrace.pEcrTrace) < 0)  ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI
      SI (fprintf(pFicTrace,"\nNBTR LGM VISU ENRG IMPR DEBG RSRV PTEC\n") <0) ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI

      /* On remplit le fichier avec des enregistrements vides : espaces */
      sprintf(FormatVariable,"%%-%d.%ds\n",
              InfoTrace.LgMsgVarTrace+LG_MSG_FIXE_TRACE,
              InfoTrace.LgMsgVarTrace+LG_MSG_FIXE_TRACE);
      iMessage = 0;
      TANTQUE iMessage < InfoTrace.NbMaxiTraces FAIRE
         SI (fprintf(pFicTrace,FormatVariable," ") < 0 ) ALORS
             Handler(BIBTRACE,ECRI_ENTETE_NOK);
         FIN_SI
         iMessage = iMessage + 1;
      FIN_TANTQUE
      FermerFichierTrace(pFicTrace);
  FIN_SI
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : LireEnteteTrace                              */
/******************************************************************************/
/* BUT : Cette procedure permet de lire l'entete du fichier trace en cours    */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* FichierTrace  | | |x| char *     | fichier DOS | Nom du fichier trace      */
/* InfoTrace     | | |x| T_ENTETE_TRACE           | Contient l'entete du      */
/*               | | | |            |             | fichier trace             */
/******************************************************************************/
/* VALEUR RETOURNEE    | entier     | OK/NOK      | fichier existant ou pas   */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : LireEnteteTrace                               */
/* Ouvrir le fichier trace et si OK alors                                     */
/*    Controle de l'entete du fichier de trace                                */
/*    Controle de coherence des valeurs lues                                  */
/*    Fermer le fichier                                                       */
/******************************************************************************/
FONCTION int LireEnteteTrace(void)
debut
   FILE *pFicTrace;    /* Descripteur de fichier */
   int cr;             /* compte rendu */

   SI (OuvreFichierTrace(FichierTrace, LECTURE, &pFicTrace) == OK) ALORS
      SI (fscanf(pFicTrace,"%d %d %x %x %x %x %d %d",
                       &InfoTrace.NbMaxiTraces,
                       &InfoTrace.LgMsgVarTrace,
                       &InfoTrace.NiveauAVisualiser,
                       &InfoTrace.NiveauAEnregistrer,
                       &InfoTrace.NiveauAImprimer,
                       &InfoTrace.NiveauADeboguer,
                       &InfoTrace.Reserve,
                       &InfoTrace.pEcrTrace) == EOF)    ALORS
         Handler(BIBTRACE,LECT_ENTETE_NOK);
      FIN_SI

      /* Controle de coherence des valeurs lues */
      SI (InfoTrace.NbMaxiTraces  > 9999) OU
         (InfoTrace.NbMaxiTraces  < 1   ) OU
         (InfoTrace.LgMsgVarTrace > 250) OU
         (InfoTrace.LgMsgVarTrace < 40)  OU
         (InfoTrace.pEcrTrace     < 1)   OU
         (InfoTrace.pEcrTrace     > InfoTrace.NbMaxiTraces) ALORS
         Handler(BIBTRACE,VAL_ENTETE_NOK);
         cr = NOK;
      SINON
         cr = OK;
      FIN_SI
      FermerFichierTrace(pFicTrace);
  SINON
      cr = NOK;
  FIN_SI
  retour cr;
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : LireNiveaux                                  */
/******************************************************************************/
/* BUT :  Lecture des niveaux de trace contenus dans le fichier trace.par     */
/* Renvoie FAUX si le fichier de parametrage n'existe pas.                    */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/******************************************************************************/
/* VALEUR RETOURNEE    | entier     |  VRAI/FAUX  | lecture/pas de lecture    */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : LireNiveaux                                   */
/*                                                                            */
/* Initalisation du tableau des consignes des traces TabCons                  */
/* Si le fichier de parametrage n'exsite pas, retour FAUX                     */
/*    memoris. dans TabCons[0] (id 0 non utilise) de l'indic. PAS_DE_CONSIGNE */
/*    retour FAUX                                                             */
/* Lecture du fichier de parametrage                                          */
/* Maj de TabCons[] en ignorant les numeros incorrects et en convertissant    */
/*     les consignes incorrects en consignes normales                         */
/* Fermeture du fichier de parametrage                                        */
/* Memorisation dans TabCons[0] de l'indic. CONSIGNE                          */
/******************************************************************************/
FONCTION int LireNiveaux()
debut
   FILE *pFichier;             /* Descripteur du fichier de parametrage */
   char ligne[LG_LIGNE];       /* Ligne du fichier */
   int  idTrace;               /* numero de trace traitee */
   char consigne=TRACE_NORMAL; /* consigne de la trace */
   char *ptr;                  /* pointeur sur la variable d'environnement */

   /* Initialisation du tableau contenant les consignes des traces */
   idTrace = 0;
   TANTQUE idTrace <= NB_MAX_TRACE FAIRE
      TabCons[idTrace]=TRACE_NORMAL;
      idTrace++;
   FIN_TANTQUE

   ptr = getenv(TRACEPAR);
   pFichier=fopen(ptr,LECTURE);
   SI pFichier == NULL ALORS
      /* Le fichier de parametrage n'existe pas */
      TabCons[0]=PAS_DE_CONSIGNE;
      retour FAUX;
   FIN_SI

   TANTQUE fgets(ligne,LG_LIGNE,pFichier)!=NULL FAIRE
      /* Ignorer toutes les differentes lignes de commentaires */
      SI ligne[0] >= '0' ET ligne[0] <='9' ALORS
         sscanf(ligne,"%4d",&idTrace); 
         consigne=ligne[6];
         /* ignorer les numeros incorrects de trace */
         SI idTrace >= 1 ET idTrace <= NB_MAX_TRACE ALORS
            LORSQUE consigne VAUT
              SOIT TRACE_NORMAL :
              SOIT TRACE_INHIBE :
              SOIT TRACE_FORCE  : TabCons[idTrace]=consigne;
                                  FIN_SOIT
              AUTRE             : /* une trace de consigne incorrecte est convertie en */
                                  /* une trace 'normal' (sans consigne particuliere) */
                                  TabCons[idTrace]=TRACE_NORMAL;
                                  FIN_SOIT
            FIN_LORSQUE
         FIN_SI
      FIN_SI
   FIN_TANTQUE

   fclose(pFichier);

   TabCons[0]=CONSIGNE;
   retour VRAI;
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : MajEnteteTrace                               */
/******************************************************************************/
/* BUT : Cette procedure permet de mettre a jour l'entete du fichier de trace */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* FichierTrace  | | |x| char *     | fichier DOS | Nom du fichier trace      */
/* InfoTrace     | | |x| T_ENTETE_TRACE           | Contient l'entete du      */
/*               | | | |            |             | fichier trace             */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
PROCEDURE MajEnteteTrace(void)
debut
   FILE *pFicTrace;    /* Descripteur de fichier                              */

   SI (OuvreFichierTrace(FichierTrace,LECT_ECR,&pFicTrace)==OK) ALORS
   SI (fprintf(pFicTrace,"%04.4d %03.3d %04.4x %04.4x %04.4x %04.4x %04.4d %04.4d",
                       InfoTrace.NbMaxiTraces,
                       InfoTrace.LgMsgVarTrace,
                       InfoTrace.NiveauAVisualiser,
                       InfoTrace.NiveauAEnregistrer,
                       InfoTrace.NiveauAImprimer,
                       InfoTrace.NiveauADeboguer,
                       InfoTrace.Reserve,
                       InfoTrace.pEcrTrace) < 0)  ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI
   SI (fprintf(pFicTrace,"\nNBTR LGM VISU ENRG IMPR DEBG RSRV PTEC\n") <0) ALORS
         Handler(BIBTRACE,ECRI_ENTETE_NOK);
      FIN_SI

      FermerFichierTrace(pFicTrace);
  FIN_SI
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : OuvreFichierTrace                            */
/******************************************************************************/
/* BUT : Ouvrir un fichier quelque-soit le mode. Si l'ouverture se passe mal, */
/*       la procedure appelle l'handler.                                      */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* NomFichier    |X| | |char*       |             | Nom fichier + repertoire  */
/* ModeOuverture |X| | |char*       |"w+","r+",...|Mode d'ouverture du fichier*/
/* pFichier      | |X| |FILE**      |             |Pointeur sur handle du     */
/*               | | | |            |             |fichier.                   */
/******************************************************************************/
/* VALEUR RETOURNEE    | int        | OK NOK      |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : OuvreFichierTrace                             */
/******************************************************************************/
FONCTION int OuvreFichierTrace(char *NomFichier,
                               char *ModeOuverture,FILE **pFichier)
debut
  int cr = OK;

  *pFichier = fopen(NomFichier,ModeOuverture);
  SI (*pFichier == NULL) ALORS
     cr = NOK;
  FIN_SI
  retour(cr);
fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : PositionnerDansFichierTrace                  */
/******************************************************************************/
/* BUT : Se positionner via un offset dans un fichier. Si cela se passe mal,  */
/*       la procedure appelle l'handler.                                      */
/*                                                                            */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* pFichier      |X| | |FILE*       |             |Handle du fichier.         */
/* Offset        |X| | |long        |             |Offset par rapport a       */
/*               | | | |            |             |l'origine.                 */
/* Origine       |X| | |int         |SEEK_CUR     |Position initiale.         */
/*               | | | |            |SEEK_SET     |                           */
/*               | | | |            |SEEK_END     |                           */
/* pFichier      |X| | |FILE*       |             |Handle du fichier.         */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : PositionnerDansFichierTrace                   */
/******************************************************************************/
PROCEDURE PositionnerDansFichierTrace(FILE *pFichier, long Offset, int Origine)
debut    

  SI fseek(pFichier,Offset,Origine) != 0 ALORS
     Handler(BIBTRACE,ERR_POSIT_FICH);
  FIN_SI   
fin


/******************************************************************************/
/************** CORPS DES SERVICES DE LA BIBLIOTHEQUE *************************/
/******************************************************************************/

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : InitTrace                                    */
/******************************************************************************/
/* BUT :  Initialise la gestion des traces :                                  */
/*              - Nom du fichier trace,                                       */
/*              - Mode RESET ou APPEND pour le fichier                        */
/* En mode RESET le fichier en remplis d'espaces.                             */
/* En mode APPEND le contenu du fichier n'est pas altere.                     */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/*NomFichierTrace|X| | | char *     |             | Nom du fichier TRACE      */
/* Mode          |X| | |  int       | TRACE_RESET | --> On efface le fichier  */
/*               | | | |            | TRACE_APPEND| --> File de l'eau         */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : InitTrace                                     */
/*                                                                            */
/******************************************************************************/
PROCEDURE InitTrace(char *nomFichierTrace,T_PTR_FONC foncNivHandler,
                    char *nomPortImprimante, int mode)
debut
   int cr;

   /* Memorisation des parametres */
   SI nomPortImprimante != NULL ALORS
      COPIER(PortImprimante,nomPortImprimante);
   FIN_SI
   SI nomFichierTrace != NULL ALORS
      COPIER(FichierTrace,nomFichierTrace);
   SINON
      COPIER(FichierTrace,"trace.txt");
   FIN_SI
   FoncNivHandler = foncNivHandler; 

   /* Lecture du fichier de parametrage des traces s'il existe */
   LireNiveaux();

   /* Lecture de l'entete du fichier de parametrage */
   cr = LireEnteteTrace();

   SI cr == NOK ALORS
      /* Valeurs par defaut de l'entete */
      InfoTrace.NbMaxiTraces = 1000;
      InfoTrace.LgMsgVarTrace = 40;
      InfoTrace.NiveauAVisualiser = 0;
      InfoTrace.NiveauAEnregistrer = 1;   /* niveau HANDLER uniquement */
      InfoTrace.NiveauAImprimer = 0;
      InfoTrace.NiveauADeboguer = 0;
      InfoTrace.Reserve = 0;  
      InfoTrace.pEcrTrace = 1;
      EcrireEnteteTrace();
   FIN_SI

   SI (mode == TRACE_RESET) ALORS
      InfoTrace.pEcrTrace = 1;
      FormaterLeFichierTrace();
   SINON
      SI (mode != TRACE_APPEND) ALORS
         Handler(BIBTRACE,MAUVAIS_MODE_TRACE);
      SINON
         SI cr == NOK ALORS
            Handler(BIBTRACE,PAS_FICH_TRACE);
         FIN_SI
      FIN_SI
   FIN_SI

fin

/******************************************************************************/
/* DESCRIPTION DE LA PROCEDURE : Trace                                        */
/******************************************************************************/
/* BUT : Traitement de la trace : VISU, ENREG, DEBUG                          */
/*       Fonction qui permet de tracer un executable lors de son execution    */
/*       et qui place ces traces dans un fichier.                             */
/******************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES         */
/*----------------------------------------------------------------------------*/
/* Nom de la     |E|S|G|  Type et   |   Valeurs   |    Signification          */
/* variable      | | | |  longueur  |  possibles  |                           */
/*----------------------------------------------------------------------------*/
/* Niveau        |X| | | int        |             | Niveau de trace           */
/* Format        |X| | | char *     |             | format de la chaine       */
/* Autres par.   |X| | | ...        |             | par. pour la chaine       */
/******************************************************************************/
/* VALEUR RETOURNEE    | Aucune     |             |                           */
/******************************************************************************/
/* ALGORITHME DE LA PROCEDURE : Trace                                         */
/*                                                                            */
/* Test de controle, on verifie que le niveau est correct                     */ 
/* Retour a l'appelant si la trace ne doit pas etre effectuee                 */
/*   qui n'appartient pas au masque de l'entete du fichier de trace           */
/* Formatage de la chaine variable recue en parametre                         */
/* SI la trace n'a pas d'id, gestion de trace sans consigne                   */
/* SI la trace est de niveau HANDLER, appeler la fonction memorisee           */
/******************************************************************************/
PROCEDURE Trace(int niveau,...)
debut
   va_list pArgument;                    /* Permet de recuperer la pile       */
   char    formatVariable[20];           /* Format calcule                    */
   char    *format;                      /* format variable recu en parametre */
   FILE    *pFicTrace;                   /* Descripteur de fichier,           */
   long    offset;                       /* Deplacement dans le fichier TRACE */
   int     lgMessage;                    /* Taille REELLE du message de trace */
   FILE * pPortImprimante;               /* fichier imprimante                */
   struct tm *dateHeure;                 /* Structure pour recuperer la date  */
   time_t    nbSecondes;                 /* Nb de seconde, calcul de la date  */
   int     idTrace;                      /* numero de la trace */
   static int flagFoncNivHandler = FAUX; /* indicateur de recursivite de      */
                                         /* l'appel a <FoncNivHandler>        */

   va_start(pArgument,niveau);

   /* Recuperation du niveau et du numero de la trace */
   SI niveau < 0 ALORS
      /* Le premier parametre corresp. en fait au num. de trace au signe pres */
      idTrace = -niveau;
      /* Le deuxieme parametre est egal au niveau */
      niveau = va_arg(pArgument,int);
   SINON
      /* La trace n'a pas de numero */
      idTrace = 0;
   FIN_SI

   /* Test de controle, on verifie que le niveau est correct */
   SI (niveau < NIVEAU_TRACE_MIN) OU (niveau > NIVEAU_TRACE_MAX) ALORS
      Handler(BIBTRACE,ERR_NIVEAU);
   FIN_SI

   /* Recuperation du format variable */
   format = va_arg(pArgument, char *);

   /* Formatage de la chaine variable recue en parametre */
   vsprintf(MsgTrace,format,pArgument);
   va_end(pArgument);

   /* Si la trace n'a pas d'id, gestion de trace sans consigne */
   SI idTrace == 0 ALORS
      TabCons[0] = PAS_DE_CONSIGNE;
   SINON
      TabCons[0] = CONSIGNE;
   FIN_SI
   va_end(pArgument);

   /* Traces */
   /* TRACEINT("\n\n----------\n"); */
   /* TRACEINT("TRACE n %d\n",idTrace); */
   /* TRACEINT("%d\n",TabCons[0]); */
   /* TRACEINT("----------\n"); */
   /* TRACEINT("niveau : %d\n",niveau); */
   /* TRACEINT("%s\n",((TabCons[0]==PAS_DE_CONSIGNE)?"PAS_DE_CONSIGNE":"CONSIGNE")); */
   /* TRACEINT("consigne = %c\n",TabCons[idTrace]); */

   /* Retour a l'appelant si la trace ne doit pas etre effectuee */
   /* qui n'appartient pas au masque de l'entete du fichier de trace */
   SI ((TabCons[0]==PAS_DE_CONSIGNE OU
       (TabCons[0]==CONSIGNE ET TabCons[idTrace]==TRACE_NORMAL)) ET
       (PAS ((InfoTrace.NiveauAVisualiser  ETBIN Masque[niveau])   OU
            (InfoTrace.NiveauAEnregistrer ETBIN Masque[niveau])    OU
            (InfoTrace.NiveauAImprimer    ETBIN Masque[niveau])))) OU
      (TabCons[0]==CONSIGNE ET TabCons[idTrace]==TRACE_INHIBE) ALORS
       
      retour;
   FIN_SI

   /* On recupere la date et l'heure */
   time(&nbSecondes);
   dateHeure = localtime(&nbSecondes);

   /* On cree le format variable pour fixer la longueur de la partie variable */
   sprintf(formatVariable,"%%-%d.%ds",InfoTrace.LgMsgVarTrace,
                                        InfoTrace.LgMsgVarTrace);
   sprintf(MsgVarTrace,formatVariable,MsgTrace);

   /* On pre-formate la trace finale avec/sans fichier selon niveauADeboguer */
   SI (Masque[niveau] ETBIN InfoTrace.NiveauADeboguer) ALORS

      SI (strlen(_Fich_) - 12) > 0 ALORS 
         /* on ne garde que les 12 derniers caracteres de _Fich_ */ 
         sprintf(_Fich_,"%s" ,(_Fich_ + strlen(_Fich_) - 12) );
      FIN_SI  
      lgMessage=sprintf(MsgTrace,"\n%02.2d/%02.2d %02d:%02d:%02d %02d %s %12.12s %4.4d",
                        dateHeure->tm_mday,
                        dateHeure->tm_mon + 1,
                        dateHeure->tm_hour,
                        dateHeure->tm_min,
                        dateHeure->tm_sec ,
                        niveau,
                        MsgVarTrace,
                        _Fich_,
                        _Ligne_);
   SINON
      lgMessage=sprintf(MsgTrace,"\n%02.2d/%02.2d %02d:%02d:%02d %02d %s %12.12s %4.4s",
                        dateHeure->tm_mday,
                        dateHeure->tm_mon + 1,
                        dateHeure->tm_hour,
                        dateHeure->tm_min,
                        dateHeure->tm_sec ,
                        niveau,
                        MsgVarTrace, 
                        "",
                        "");
   FIN_SI

   /* Traitement de la visualisation */
   SI (Masque[niveau] ETBIN InfoTrace.NiveauAVisualiser) ALORS
      printf("%s",MsgTrace);
   FIN_SI

   /* Traitement des enregistrements */
   SI (Masque[niveau] ETBIN InfoTrace.NiveauAEnregistrer) OU
      (TabCons[idTrace] == TRACE_FORCE) ALORS
      LireEnteteTrace(); 
      SI (OuvreFichierTrace(FichierTrace,LECT_ECR,&pFicTrace)==OK) ALORS
         offset = (long)TAILLE_ENTETE + ((long)InfoTrace.pEcrTrace-1L)*
                                         (long)lgMessage;
         PositionnerDansFichierTrace(pFicTrace,offset,SEEK_SET);
         fprintf(pFicTrace,"%s",MsgTrace);
         FermerFichierTrace(pFicTrace);
      FIN_SI

      /* On calcule le numero du prochain enregistrement */
      InfoTrace.pEcrTrace++;
      SI InfoTrace.pEcrTrace > InfoTrace.NbMaxiTraces ALORS
         InfoTrace.pEcrTrace = 1;
      FIN_SI
      EcrireEnteteTrace();
   FIN_SI

   /* Traitement de l'impression */
   /* pour eviter un warning --> */  pPortImprimante=NULL;
   SI (Masque[niveau] ETBIN InfoTrace.NiveauAImprimer) ET
      (PortImprimante!=NULL) ALORS
      OuvreFichierTrace(PortImprimante,CREAT_APPEND,&pPortImprimante);
      fprintf(pPortImprimante,"%s",MsgTrace);
      FermerFichierTrace(pPortImprimante);
   FIN_SI

   /* Si la trace est de niveau HANDLER, appeler la fonction memorisee */
   SI niveau == NIV_HANDLER ET FoncNivHandler != NULL ALORS
      SI flagFoncNivHandler == FAUX ALORS
         flagFoncNivHandler = VRAI;
         (FoncNivHandler)();
         flagFoncNivHandler = FAUX;
      FIN_SI
   FIN_SI
fin

/******************************* F I N ****************************************/
