/*****************************************************************************/
/*                                                                           */
/* DESCRIPTION DE L'UCS (UNITE COMPILABLE SEPAREMENT) : SPEC.C               */
/*                                                                           */
/*****************************************************************************/
#include "bingo.h"

/*****************************************************************************/
/*                        CONSTANTES ET TYPES                                */
/*****************************************************************************/

/* les lignes de commentaire commenent par le caractre suivant */
#define COMMENTAIRE        '/'

/* longueur maximum d'une ligne */

/* definition des separateurs */
#define ISSEPARATEUR(a)         ((a)==9   OU (a)==' ')
#define SEPARATEUR(a)       ((a)==';')

/* Les libelles (mots cls) dans les fichiers */
#define NB_LIBELLE          15

#define LIB_ELEVE           "ELEVE"

#define LIB_BREVET          "BREVET"

#define LIB_TITRE           "TITRE"
#define LIB_CATEGORIE       "CATEGORIE"

#define LIB_PASSE           "PASSE"
#define LIB_MODE            "MODE"

#define LIB_AUTEUR          "AUTEUR"

#define LIB_RESP            "RESP"
#define LIB_TRONC           "TRONC"

#define LIB_COULEUR         "COULEUR"

#define LIB_USER            "USER"

#define LIB_NIVEAU          "NIVEAU"


#define PAS_ID              -1
#define ID_ELEVE            1

#define ID_BREVET           2

#define ID_TITRE            3
#define ID_CATEGORIE        4

#define ID_PASSE            5
#define ID_MODE             6

#define ID_AUTEUR           7

#define ID_RESP            11

#define ID_TRONC           12

#define ID_COULEUR         13

#define ID_USER            14

#define ID_NIVEAU          15



static T_LIBID Cle[NB_LIBELLE]={
	{LIB_ELEVE,ID_ELEVE},{LIB_BREVET,ID_BREVET},{LIB_TITRE,ID_TITRE},
	{LIB_CATEGORIE,ID_CATEGORIE},
	{LIB_PASSE,ID_PASSE},{LIB_MODE,ID_MODE},
	{LIB_AUTEUR,ID_AUTEUR},{LIB_NIVEAU,ID_NIVEAU},
	{LIB_RESP,ID_RESP},
	{LIB_TRONC,ID_TRONC},
	{LIB_COULEUR,ID_COULEUR},{LIB_USER,ID_USER}
};


/* Rq.: le deuxime champ Id est "gal"  l'indice du tableau. */
/* La structure T_LIBID est utilise pour pouvoir utiliser */
/* la fonction "IdentMot" */
T_LIBID Categorie[NB_MAX_CATEGORIE]={
	{"",0},{"",1},{"",2},{"",3},{"",4},{"",5},{"",6},{"",7},
	{"",8},{"",9},{"",10},{"",11},{"",12},{"",13}
};

/********************************* INDEX *************************************/

/* definition des index des messages Syntaxe Erreur   */
/* (Cf. Tableau TabSyntaxeErreur dans module inter.c) */


/*****************************************************************************/
/* DESCRIPTION DE LA FONCTION : ChercherMot                                  */
/*****************************************************************************/
/* BUT : Fonction de recherche d'un mot dans une chaine                      */
/* Un mot est un ensemble de caracteres precedes d'un ou plusieurs           */
/* separateurs (ESPACE, TAB, '=') et suivi d'un separateur ou d'une */
/* fin de ligne ('\0' ou '\n').                                              */
/* Un pointeur sur la position de depart pour la recherche est passe en      */
/* parametre et est mis a jour au fur et a mesure de la recherche            */
/* il doit etre inferieur a LG_LIGNE                                         */
/* Utilise la macro ISSEPARATEUR qui teste si un caractere est separateur    */
/*****************************************************************************/
/*    ARGUMENTS D'ENTREE OU DE SORTIE ET VARIABLES GLOBALES MODIFIEES        */
/*****************************************************************************/
/* ch            |x| | |  chaine    |             |chaine de recherche       */
/* poffset       |x|x| | @ entier   |             |position dans la chaine   */
/* mot           | |x| | chaine     |             |mot trouve                */
/*---------------------------------------------------------------------------*/
/* Valeur retrouvee    | entier     |OK/NOK       | OK si mot trouve         */
/*****************************************************************************/
FONCTION int ChercherMot(char *ch,int *poffset,char *mot)
debut
  int deb=0;

  /* verification des bornes de l'indice */
  SI *poffset<0 OU *poffset>=LG_LIGNE ALORS
    mot[0]='\0';
    retour NOK;
  FIN_SI

  /* elimination des separateurs qui precedent un mot */
  TANTQUE ISSEPARATEUR(ch[*poffset]) FAIRE
    (*poffset)++;
  FIN_TANTQUE

  /* cas de la fin de chaine: pas de mot trouve */
  SI ch[*poffset]=='\0' OU ch[*poffset]=='\n' ALORS
    mot[0]='\0';
    retour NOK;
  FIN_SI

  /* recopie du mot trouve jusqu'a un autre separateur */
  deb=*poffset;
  TANTQUE !(ISSEPARATEUR(ch[*poffset]) OU
            ch[*poffset]=='\0' OU
            ch[*poffset]=='\n') FAIRE
    mot[(*poffset)-deb]=ch[*poffset];
    (*poffset)++;
  FIN_TANTQUE

  /* ecrire fin de chaine (0) en fin de Mot */
  mot[(*poffset)-deb]='\0';
  retour OK;
fin

/*****************************************************************************/
/* DESCRIPTION DE LA FONCTION : MemoFinLigne                                 */
/*****************************************************************************/
FONCTION int MemoFinLigne(char *ch,int *poffset,char *mot)
debut
  int deb=0;

  /* verification des bornes de l'indice */
  SI *poffset<0 OU *poffset>=LG_LIGNE ALORS
    mot[0]='\0';
    retour NOK;
  FIN_SI

  /* elimination des separateurs qui precedent un mot */
  TANTQUE ISSEPARATEUR(ch[*poffset]) FAIRE
    (*poffset)++;
  FIN_TANTQUE

  /* cas de la fin de chaine: pas de mot trouve */
  SI ch[*poffset]=='\0' OU ch[*poffset]=='\n' ALORS
    mot[0]='\0';
    retour NOK;
  FIN_SI

  /* recopie du mot trouve jusqu'a la fin de la ligne */
  deb=*poffset;
  TANTQUE !(ch[*poffset]=='\0' OU
            ch[*poffset]=='\n') FAIRE
    mot[(*poffset)-deb]=ch[*poffset];
    (*poffset)++;
  FIN_TANTQUE

  /* ecrire fin de chaine (0) en fin de Mot */
  mot[(*poffset)-deb]='\0';
  retour OK;
fin

/*****************************************************************************/
/* DESCRIPTION DE LA FONCTION : LitEntier                                    */
/*****************************************************************************/
FONCTION int LitEntier(char *ch,int *poffset,int *pvaleur)
debut
  char Mot[LG_LIGNE];

  SI ChercherMot(ch,poffset,Mot)==NOK ALORS
     retour NOK;
  SINON
     *pvaleur=atoi(Mot);
     retour OK;
  FIN_SI
fin

/*****************************************************************************/
/* DESCRIPTION DE LA FONCTION : LitLong                                      */
/*****************************************************************************/
FONCTION int LitEntierLong(char *ch,int *poffset,unsigned long *pvaleur)
debut
  char Mot[LG_LIGNE];

  SI ChercherMot(ch,poffset,Mot)==NOK ALORS
     retour NOK;
  SINON
     *pvaleur=atol(Mot);
     retour OK;
  FIN_SI
fin

/*****************************************************************************/

FONCTION int IdentMot(T_LIBID *tab,int taille,char *mot)
debut
  int Indice=0;

  SI mot[0]=='\0' ALORS
     retour PAS_ID;
  FIN_SI

  /* boucle sur le tableau */
  TANTQUE  Indice<taille FAIRE

    /* si mot = libelle du tableau[indice] alors retourner son ident. */
    SI strncmp(mot,tab[Indice].Libelle,strlen(mot))==0 ALORS
       retour(tab[Indice].Id);
    FIN_SI
    Indice++;

  /* Fin boucle */
  FIN_TANTQUE

  /* mot pas trouve, retour PAS_ID */
  retour(PAS_ID);
fin

/* ------------------------------------------------------------------------ */
/*                          LA BASE DE DONNEES                              */
/* ------------------------------------------------------------------------ */
PROCEDURE ArchiveChangement(int idEnfant, int idBrevet, int emetteur, char *pref)
debut
  char repertoire[LG_PATH+1];
  FILE *ptFic;
  char chDate[30];
  char surnom[LG_SURNOM_ELEVE+1];
  char nomEmetteur[LG_SURNOM_ELEVE+1];
  char nom[LG_TITRE_BREVET+1];
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+1];

  SI idBrevet>=0 ALORS
     nom[0]='\0';
     comment1[0]='\0';
     comment2[0]='\0';
     LitInfosBrevet(idBrevet,nom,comment1,comment2);
  SINON
     COPIER(nom,"");
  FIN_SI
  SI idEnfant>=0 ALORS
     COPIER(surnom,L_SURNOMELEVE(idEnfant));
  SINON
     COPIER(surnom,"ARBRE");
  FIN_SI

  /* enregistrement dans le fichier derniers changements */
  L_REPDONNEES(repertoire);
  chdir(repertoire);

  ptFic=fopen(FIC_NOUVELLES,"at");
  SI ptFic!=NULL ALORS
     SI emetteur==ACCES_INSTIT ALORS
	COPIER(nomEmetteur,"Responsable");
     SINON
	SI emetteur!=-1 ALORS
	   COPIER(nomEmetteur,L_SURNOMELEVE(emetteur));
	SINON
	   COPIER(nomEmetteur,"Bingo");
	FIN_SI
     FIN_SI

     AffichDate2(chDate);
     fprintf(ptFic,"%.4s%.10s:%.36s... %.8s %s\n",pref,surnom,nom,nomEmetteur,chDate);
     fclose(ptFic);
  FIN_SI
  L_REPCOURANT(repertoire);
  chdir(repertoire);
fin

/*FONCTION int TestBD(int affichage)
debut
  char repertoire[LG_PATH+1];

  retour OK;    BUG dans chdir !  cause de Windows XP ?!?

  L_REPDONNEES(repertoire);

  SI repertoire[0]!='\0' ALORS
     SI chdir(repertoire)==0 VRAI ALORS
	L_REPCOURANT(repertoire);
	chdir(repertoire);
	retour OK;
     FIN_SI
  FIN_SI
  SI affichage==VRAI ALORS
     Message(ATTRTITLE,NON_INIT,"AUCUNE BASE DE DONNEES OUVERTE ! ");
     retour NOK;
  FIN_SI
fin */

/*---------------------------------------------------*/

FONCTION int OuvertureBase()
debut
  char repertoire[LG_PATH+1];
  int cr,crConfig;
  FILE *ptFic;
  char chDate[30];

  cr=NOK;
  L_REPDONNEES(repertoire);
  SI repertoire[0]!='\0' ET chdir(repertoire) >=0 ALORS
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     crConfig=LectureConfig();
     SI (cr=InitBrevets()) == OK ALORS
	SI(cr=InitEleves()) == OK ALORS
	   LORSQUE crConfig VAUT
	     SOIT CONV12 : Conv12();
			   FIN_SOIT;
	     SOIT CONV20 : Conv20();
			   FIN_SOIT;
	   FIN_LORSQUE

	   L_REPDONNEES(repertoire);
	   cr=FICHIER_OUVERT;
	   /* initialisation des valeurs par dfaut */
	   LargeurTronc=max(3,L_NBBREVETSDANSARBRE()/20);

	SINON
	   cr=FICHIER_ERREUR;
	FIN_SI
     SINON
	cr=FICHIER_ERREUR;
     FIN_SI
   SINON
     cr=FICHIER_ABSENT;
   FIN_SI

   SI cr!=FICHIER_OUVERT ALORS
     E_NBELEVES(0);
     E_NBBREVETS(0);
     SI cr==FICHIER_ABSENT ALORS
	COPIER(nomBD,"AUCUNE BASE DE DONNEES OUVERTE");
     FIN_SI
     E_REPDONNEES(DOSSIER_DEFAUT);
   FIN_SI

   /* Initialisation du fichier "Derniers changements" s'il n'existe pas */

   L_REPDONNEES(repertoire);
   chdir(RepDonnees);
   ptFic=fopen(FIC_NOUVELLES,"rt");
   SI ptFic==NULL ALORS
      AffichDate(chDate);
      SI fopen(FIC_NOUVELLES,"wt")!=NULL ALORS
	 fprintf(ptFic,"%s\n\n",chDate);
	 fclose(ptFic);
      FIN_SI
   SINON
      fclose(ptFic);
   FIN_SI

   L_REPCOURANT(repertoire);
   chdir(repertoire);


   retour cr;
fin

/* ------------------------------------------------------------------------ */
/*               LE FICHIER CONTENANT LES BREVETS                           */
/* ------------------------------------------------------------------------ */

FONCTION int LectFicListeBrevets(void)
debut
  FILE *ptFic;
  char ligne[LG_LIGNE];
  char mot[LG_LIGNE];
  char nomBrevet[LG_TITRE_BREVET+1];
  int deb,colonne=0,noLigne=0,identif,idBrevet=0,idAuteur=0;
  char nomFic[LG_FIC_EXT+1];
  char repertoire[LG_PATH+1];
  int arretLectureFichier=FAUX;
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+1];
  int ancienneVersion=FAUX;

  /* lecture du fichier des brevets */
  L_FICBREVETS(nomFic);
  L_REPDONNEES(repertoire);
  chdir(repertoire);

  ptFic=fopen(nomFic,"rt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER (SPEC LectFicListeBrevets)");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour NOK;
  FIN_SI

  /* Tant que (pas fin de fichier) faire */
  TANTQUE fgets(ligne,LG_LIGNE,ptFic)!=NULL ET arretLectureFichier==FAUX FAIRE
    colonne=0;

    /* ignorer les commentaires */
    SI ligne[0]!=COMMENTAIRE ALORS
      /* ignorer les lignes vides, lecture du premier mot */
      SI ChercherMot(ligne,&colonne,mot)!=NOK ALORS

        /* rechercher l'identificateur du mot */
        identif=IdentMot(Cle,NB_LIBELLE,mot);
	LORSQUE (identif) VAUT
	    SOIT ID_BREVET : SI idBrevet<NB_MAX_BREVETS ALORS
				idBrevet++;
				E_NBBREVETS(L_NBBREVETS()+1);
				/* valeurs par dfaut */
				E_IDCATEGORIE(idBrevet,AUCUNE_CATEGORIE);
				E_IDAUTEUR(idBrevet,AUCUN_AUTEUR);
			    SINON
				arretLectureFichier=VRAI;
				Message(13,ATTRTITLE,-1,"\n\nNombre de brevets limits  %d !",NB_MAX_BREVETS);
			    FIN_SI
			    FIN_SOIT
	    SOIT ID_TITRE : /* ancien format de donnes (avant la version 3.0) */
			    /* conversion */
			    SI idBrevet<NB_MAX_BREVETS ALORS
			       idBrevet++;
			       deb=colonne;
			       TANTQUE (!(ligne[colonne]=='\0' OU
					ligne[colonne]=='\n')) ET
					((colonne-deb) < LG_TITRE_BREVET) FAIRE
				    /* on ne prend pas en compte les premiers espaces */
				    SI ligne[colonne]==' ' ET deb==colonne ALORS
				       deb++;
				    SINON
				       nomBrevet[colonne-deb]=ligne[colonne];
				    FIN_SI
				    colonne++;
				FIN_TANTQUE
				nomBrevet[colonne-deb]='\0';

				LitInfosBrevetAvantV3(idBrevet, comment1, comment2);
				SauveInfosBrevet(idBrevet, nomBrevet, comment1, comment2);

				E_NBBREVETS(L_NBBREVETS()+1);
				/* valeurs par dfaut */
				E_IDCATEGORIE(idBrevet,AUCUNE_CATEGORIE);
			    SINON
				arretLectureFichier=VRAI;
				Message(13,ATTRTITLE,-1,"\n\nNombre de brevets limits  %d !",NB_MAX_BREVETS);
			    FIN_SI
			    ancienneVersion=VRAI;
                            FIN_SOIT
	SOIT ID_CATEGORIE : MemoFinLigne(ligne,&colonne,mot);
			    mot[LG_CATEGORIE]='\0';
			    identif=IdentMot(Categorie,NB_MAX_CATEGORIE,mot);
			    SI identif!=PAS_ID ALORS
			       E_IDCATEGORIE(idBrevet,identif);
			    SINON
			       E_IDCATEGORIE(idBrevet,AUCUNE_CATEGORIE);
			    FIN_SI
			    FIN_SOIT

	 SOIT ID_AUTEUR   : LitEntier(ligne,&colonne,&idAuteur);
			    E_IDAUTEUR(idBrevet,idAuteur);
			    FIN_SOIT
           SOIT ID_ELEVE  :
                             SyntaxeErreur(ID_SELIB,nomFic,noLigne+1);
			     fclose(ptFic);
			     L_REPCOURANT(repertoire);
			     chdir(repertoire);
			     getch();
			     Quitter(0);
			     FIN_SOIT
	  AUTRE            : FIN_SOIT
        FIN_LORSQUE
      FIN_SI
    FIN_SI
    noLigne++;
  FIN_TANTQUE

  /* fermeture du fichier */
  fclose(ptFic);

  /* retour dans le repertoire courant */
  L_REPCOURANT(repertoire);
  chdir(repertoire);

  SI ancienneVersion==VRAI ALORS
     EnregFicBrevets();
  FIN_SI

  retour OK;

fin

/* ********************************* */

FONCTION int InitBrevets()
debut
  int i,cr;

  /* Initialisation des brevets */
  E_NBBREVETS(0);
  E_NBCATEGORIE(0);
  i=0;
  TANTQUE i<NB_MAX_BREVETS FAIRE
     E_IDCATEGORIE(i,AUCUNE_CATEGORIE);
     E_IDAUTEUR(i,AUCUN_AUTEUR);
     i++;
  FIN_TANTQUE

  /* Lecture du fichier des catgories */
  LectureFicCat();

  /* Lecture du fichier des brevets */
  cr=LectFicListeBrevets();

  AffichBandeau();

  retour cr;

fin

FONCTION int EnregFicBrevets()
debut
  char nomFic[LG_FIC_EXT+1];
  FILE *ptFic;
  int i;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  L_FICBREVETS(nomFic);

  ptFic=fopen(nomFic,"wt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour NOK;
  FIN_SI

  fputs("// Liste des brevets\n\n",ptFic);

  i=1;
  TANTQUE i<=L_NBBREVETS() FAIRE
     fprintf(ptFic,"%s %d\n",LIB_BREVET,i);
     SI L_IDCATEGORIE(i)!=AUCUNE_CATEGORIE ALORS
	fprintf(ptFic,"%s %s\n",LIB_CATEGORIE,Categorie[L_IDCATEGORIE(i)].Libelle);
     FIN_SI
     SI L_IDAUTEUR(i)!=AUCUN_AUTEUR ALORS
	fprintf(ptFic,"%s %d %s\n",LIB_AUTEUR,L_IDAUTEUR(i),L_SURNOMELEVE(L_IDAUTEUR(i)));
     FIN_SI
     i++;
  FIN_TANTQUE

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour OK;
fin

/* ********************************* */
PROCEDURE SauveInfosBrevet(int idBrevet, char *titre, char *comment1, char *comment2)
debut
  FILE *ptFic;
  char nomFic[12];
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  itoa(idBrevet,nomFic,10);
  strcat(nomFic,".TXT");
  ptFic=fopen(nomFic,"wt");
  SI ptFic!=NULL ALORS
     fprintf(ptFic,"%s\n%s\n%s",titre,comment1,comment2);
     fclose(ptFic);
  FIN_SI

  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin


/* ********************************* */
PROCEDURE LitInfosBrevetAvantV3(int idBrevet, char *comment1, char *comment2)
debut
  FILE *ptFic;
  char nomFic[12];
  int i, long1, long2;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  long1=strlen(comment1);
  long2=strlen(comment2);
  itoa(idBrevet,nomFic,10);
  strcat(nomFic,".TXT");
  ptFic=fopen(nomFic,"rt");
  SI ptFic!=NULL ALORS
     fgets(comment1,LG_LIGNE,ptFic);
     fgets(comment2,LG_LIGNE,ptFic);
     i=strlen(comment1);
     TANTQUE i<long1 FAIRE
	comment1[i]=' ';
	i++;
     FIN_TANTQUE
     comment1[i]='\0';
     i=strlen(comment2);
     TANTQUE i<long2 FAIRE
	comment2[i]=' ';
	i++;
     FIN_TANTQUE
     comment2[i]='\0';
     fclose(ptFic);
  FIN_SI

  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin

/* ********************************* */
PROCEDURE LitInfosBrevet(int idBrevet, char *titre, char *comment1, char *comment2)
debut
  FILE *ptFic;
  char nomFic[12];
  int longtitre, long1, long2;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  itoa(idBrevet,nomFic,10);
  strcat(nomFic,".TXT");
  ptFic=fopen(nomFic,"rt");
  SI ptFic!=NULL ALORS
     fgets(titre,LG_LIGNE,ptFic);
     fgets(comment1,LG_LIGNE,ptFic);
     fgets(comment2,LG_LIGNE,ptFic);
     longtitre=strlen(titre);
     long1=strlen(comment1);
     long2=strlen(comment2);

     SI titre[longtitre-1]=='\n' ALORS
	titre[longtitre-1]='\0';
     FIN_SI
     titre[longtitre]='\0';

     SI comment1[long1-1]=='\n' ALORS
	comment1[long1-1]='\0';
     FIN_SI
     comment1[long1]='\0';

     SI comment2[long2-1]=='\n' ALORS
	comment2[long2-1]='\0';
     FIN_SI
     comment2[long2]='\0';

     fclose(ptFic);
  FIN_SI

  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin


/* ********************************* */

PROCEDURE ModifBrevetDansFicBrevet(int idBrevet, int idCat, int idAuteur)
debut
  SI idBrevet>0 ALORS
     InitBrevets(); /* dernire lecture des brevets */
     SI idBrevet>L_NBBREVETS() ALORS
	E_NBBREVETS(idBrevet);
     FIN_SI
     E_IDCATEGORIE(idBrevet,idCat);
     E_IDAUTEUR(idBrevet,idAuteur);
     EnregFicBrevets();
  FIN_SI
fin


/*PROCEDURE AjouBrevetDansFicBrevet(int idNouveauBrevet)
debut
  int nbBrevets;

  nbBrevets=idNouveauBrevet;
  E_NBBREVETS(nbBrevets);
  EnregFicBrevets();
fin                 */

/* ------------------------------------------------------------------------ */
/*               LE FICHIER CONTENANT LES CATEGORIES                        */
/* ------------------------------------------------------------------------ */

PROCEDURE LectureFicCat(void)
debut
  FILE *PtFic;
  char Ligne[LG_LIGNE];
  char Mot[LG_LIGNE];
  int Colonne=0,NoLigne=0,Identif;
  int idCat=0;
  char nomFic[LG_FIC_EXT+1];
  char repertoire[LG_PATH+1];

  /* lecture du nom du fichier */
  L_FICCATEGORIE(nomFic);
  L_REPDONNEES(repertoire);
  chdir(repertoire);

  /* ouverture du fichier */
  PtFic=fopen(nomFic,"rt");
  SI PtFic==NULL ALORS
     /* pas de fichier de catgorie - rappel : optionnel */
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  /* Tant que (pas fin de fichier) faire */
  TANTQUE fgets(Ligne,LG_LIGNE,PtFic)!=NULL FAIRE
    Colonne=0;

    /* ignorer les commentaires */
    SI Ligne[0]==COMMENTAIRE ALORS
    SINON
      /* ignorer les lignes vides, lecture du premier mot */
      SI ChercherMot(Ligne,&Colonne,Mot)==NOK ALORS
      SINON

         /* rechercher l'identificateur du mot */
         Identif=IdentMot(Cle,NB_LIBELLE,Mot);
         /* si mot inconnu alors Handler */
         SI Identif==PAS_ID ALORS
	    SyntaxeErreur(ID_SELIB,nomFic,NoLigne+1);
				fclose(PtFic);
	    L_REPCOURANT(repertoire);
	    chdir(repertoire);
	    retour;
         SINON

            /* verification des parametres de l'identificateur */
            LORSQUE (Identif) VAUT
	       SOIT ID_CATEGORIE : MemoFinLigne(Ligne,&Colonne,Mot);
				   Mot[LG_CATEGORIE]='\0';
				   SI idCat<NB_MAX_CATEGORIE ALORS
				      sprintf(Categorie[idCat].Libelle,"%-14s",Mot);
				      Categorie[idCat].Libelle[strlen(Categorie[idCat].Libelle)]='\0';
				      Categorie[idCat].Id=idCat;
				      idCat++;
				      E_NBCATEGORIE(idCat);
				   FIN_SI
                                FIN_SOIT
               AUTRE          :
				SyntaxeErreur(ID_SELIB,nomFic,NoLigne+1);
            FIN_LORSQUE
         FIN_SI
      FIN_SI
    FIN_SI
    NoLigne++;
  FIN_TANTQUE

  /* fermeture du fichier */
  fclose(PtFic);

  /* retour au repertoire courant */
  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin

/* **************************** */

PROCEDURE EnregFicCat(void)
debut
  char nomFic[LG_FIC_EXT+1];
  FILE *ptFic;
  int i,nbCat;
  char repertoire[LG_PATH+1];
  char cat[LG_CATEGORIE+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  L_FICCATEGORIE(nomFic);

  ptFic=fopen(nomFic,"wt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  fprintf(ptFic,"// mot cl : %s\n",LIB_CATEGORIE);
  fprintf(ptFic,"//\n");

  i=0;
  nbCat=0;
  TANTQUE i<NB_MAX_CATEGORIE FAIRE
     COPIER(cat,Categorie[i].Libelle);
     SI strtok(cat," ")!=NULL ALORS
	fprintf(ptFic,"%s    %s\n",LIB_CATEGORIE,Categorie[i].Libelle);
	nbCat++;
     FIN_SI
     i++;
  FIN_TANTQUE

  E_NBCATEGORIE(nbCat);

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour;
fin

/* **************************** */

PROCEDURE ModifFicCategorie()
debut
  /* enregistrement des nouvelles catgories */
  EnregFicCat();
  /* Reinitialisation/lecture des donnees brevets pour maj des ndices des catgories */
  InitBrevets();
fin

/* ------------------------------------------------------------------------ */
/*               LE FICHIER CONTENANT LA LISTE DES ELEVES                   */
/* ------------------------------------------------------------------------ */

FONCTION int LectFicListeEleves(void)
debut
  FILE *PtFic,*eleveFic;
  char Ligne[LG_LIGNE];
  char Mot[LG_LIGNE];
  int Colonne=0,NoLigne=0,Identif,idEleve;
  char nomFic[LG_FIC_EXT+1];
  char repertoire[LG_PATH+1];

  L_FICELEVES(nomFic);
  L_REPDONNEES(repertoire);
  chdir(repertoire);

  /* ouverture du fichier */
  PtFic=fopen(nomFic,"rt");
  SI PtFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER LISTE ELEVES");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour FAUX;
  FIN_SI

  /* Tant que (pas fin de fichier) faire */
  TANTQUE fgets(Ligne,LG_LIGNE,PtFic)!=NULL FAIRE
    Colonne=0;

    /* ignorer les commentaires */
    SI Ligne[0]!=COMMENTAIRE ALORS
      /* ignorer les lignes vides, lecture du premier mot */
      SI ChercherMot(Ligne,&Colonne,Mot)!=NOK ALORS

        /* rechercher l'identificateur du mot */
        Identif=IdentMot(Cle,NB_LIBELLE,Mot);
	LORSQUE (Identif) VAUT

	    SOIT ID_ELEVE  : idEleve=L_NBELEVES();
			     SI idEleve==NB_MAX_ELEVES ALORS
				fclose(PtFic);
				L_REPCOURANT(repertoire);
				chdir(repertoire);
				Message(13,ATTRTITLE,NON_INIT,"Le nombre d'enfants est limit  %d !",NB_MAX_ELEVES);
				retour VRAI;
			     FIN_SI

			     /* prnom */
			     SI ChercherMot(Ligne,&Colonne,Mot)==OK ET
				strlen(Mot)<=LG_NOM_ELEVE ALORS
				 E_PRENOMELEVE(idEleve,Mot);
			     SINON
				   SyntaxeErreur(ID_SEPE,nomFic,NoLigne+1);
                                   fclose(PtFic);
                                   retour FAUX;
			     FIN_SI

			     /* nom */
			     SI ChercherMot(Ligne,&Colonne,Mot)==OK ET
				strlen(Mot)<=LG_NOM_ELEVE ALORS
				E_NOMELEVE(idEleve,Mot);
			     SINON
				SyntaxeErreur(ID_SENE,nomFic,NoLigne+1);
				fclose(PtFic);
				retour FAUX;
			     FIN_SI

			     /* surnom */
			     SI ChercherMot(Ligne,&Colonne,Mot)==OK ET
				strlen(Mot)<=LG_SURNOM_ELEVE ALORS
				E_SURNOMELEVE(idEleve,Mot);
			     SINON
				SyntaxeErreur(ID_SESE,nomFic,NoLigne+1);
				fclose(PtFic);
				retour FAUX;
			     FIN_SI

			     /* mot de passe */
			     SI ChercherMot(Ligne,&Colonne,Mot)==OK ET
				strlen(Mot)<=LG_MOT_PASSE ALORS
				COPIER(Mot,strupr(Mot));
				E_MOTPASSE(idEleve,Mot);
			     SINON
				SyntaxeErreur(ID_SEPS,nomFic,NoLigne+1);
				fclose(PtFic);
				retour FAUX;
			     FIN_SI

			     /* nom du fichier */
			     SI ChercherMot(Ligne,&Colonne,Mot)==OK ET
				strlen(Mot)<=LG_FIC_EXT ALORS
                                   /* ouverture du fichier */
                                   eleveFic=fopen(Mot,"rt");
                                   SI eleveFic==NULL ALORS
				      Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER ELEVE %s",Mot);
				      fclose(PtFic);
				      L_REPCOURANT(repertoire);
				      chdir(repertoire);
                                      retour FAUX;
				   SINON
                                      fclose(eleveFic);
				   FIN_SI
				   E_NOMFIC(idEleve,Mot);
				   E_NBELEVES(idEleve+1);
			      SINON
				   SyntaxeErreur(ID_SEFE,nomFic,NoLigne+1);
                                   fclose(PtFic);
                                   retour FAUX;
			      FIN_SI

			      FIN_SOIT

	  AUTRE            : SI Identif!=ID_NIVEAU ALORS
				SyntaxeErreur(ID_SELIB,nomFic,NoLigne+1);
				fclose(PtFic);
				retour FAUX;
			     FIN_SI
			     FIN_SOIT
        FIN_LORSQUE
      FIN_SI
    FIN_SI
    NoLigne++;
  FIN_TANTQUE

  /* fermeture du fichier */
  fclose(PtFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour VRAI;

fin

/* ********************************* */

FONCTION int InitEleves()
debut
  int i,cr=OK;
  char repertoire[LG_PATH+1];

  /* Initialisation des donnes des lves */
  E_NBELEVES(0);
  i=0;
  TANTQUE i<NB_MAX_ELEVES FAIRE
     E_NBBREVETSELEV(i,0);
     i++;
  FIN_TANTQUE

  i=1;
  TANTQUE i<=NB_MAX_BREVETS FAIRE
     E_NBDETENTEURS(i,0);
     i++;
  FIN_TANTQUE

  SI LectFicListeEleves() == FAUX ALORS
     retour NOK;
  FIN_SI

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  /* Lecture des fichiers eleves */
  i=0;cr=OK;
  TANTQUE i<L_NBELEVES() ET cr==OK FAIRE
     cr=LectFicEleve(i);
     i++;
  FIN_TANTQUE

  /* Memorisation du nombre de brevets dans l'arbre */
  E_NBBREVETSDANSARBRE(0);
  i=1;
  TANTQUE i<=L_NBBREVETS() FAIRE
     SI L_NBDETENTEURS(i)>0 ALORS
	E_NBBREVETSDANSARBRE(L_NBBREVETSDANSARBRE()+1);
     FIN_SI
     i++;
  FIN_TANTQUE

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  AffichBandeau();

  E_NBREUSSITES(CalculReussites());

  retour cr;
fin


/* ********************************* */

PROCEDURE EnregFicEleves(int idEleveSupp)
debut
  char nomFic[LG_FIC_EXT+1];
  FILE *ptFic;
  int i;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  L_FICELEVES(nomFic);

  ptFic=fopen(nomFic,"wt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  fprintf(ptFic,"// mot cl : %s\n",LIB_ELEVE);
  fprintf(ptFic,"//\n");
  fprintf(ptFic,"//            Prnom             Nom          SurNom           Passe         Fichier\n");
  fprintf(ptFic,"//\n");

  i=0;
  TANTQUE i<L_NBELEVES() FAIRE
     SI i!=idEleveSupp ALORS
	fprintf(ptFic,"%s%16s%16s%16s%16s%16s\n",LIB_ELEVE,L_PRENOMELEVE(i),
		   L_NOMELEVE(i),L_SURNOMELEVE(i),L_MOTPASSE(i),L_NOMFIC(i));
     SINON
	unlink(L_NOMFIC(i));
     FIN_SI
     i++;
  FIN_TANTQUE

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour;
fin

/* ********************************* */

PROCEDURE ModifMotPasseDansFicEleves (int idEnfant, char *motPasse)
debut
  InitEleves();
  E_MOTPASSE(idEnfant,motPasse);
  EnregFicEleves(-1);
  ArchiveChangement(idEnfant, -1, idEnfant, "P ");
fin

/* ------------------------------------------------------------------------ */
/*               LES FICHIERS ELEVES CONTENANT LES BLASONS                  */
/* ------------------------------------------------------------------------ */

PROCEDURE LectureDesPTs(void)
debut
  int eleve,j;

  FILE *ptFic;
  char ligne[LG_LIGNE];
  char mot[LG_LIGNE];
  int i, colonne, identif, numBrevet,noLigne=0;
  char repertoire[LG_PATH+1];

  char nomPT[LG_FIC_EXT+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  i=0;
  TANTQUE i<NB_MAX_ITEM_PT*NB_MAX_ELEVES+1 FAIRE
     PT[i]=NON_INIT;
     i++;
  FIN_TANTQUE

  eleve=0;i=0;
  TANTQUE eleve<=L_NBELEVES() FAIRE

     COPIER(nomPT,L_NOMFIC(eleve));
     j=0;
     TANTQUE nomPT[j]!='.' ET nomPT[j]!='\0' FAIRE
	j++;
     FIN_TANTQUE
     nomPT[j]='\0';
     strcat(nomPT,".pt");

     ptFic=fopen(nomPT,"rt");
     SI ptFic!=NULL ALORS

	TANTQUE fgets(ligne,LG_LIGNE,ptFic)!=NULL FAIRE
	   colonne=0;

	   SI ligne[0]!=COMMENTAIRE ALORS
	      SI ChercherMot(ligne,&colonne,mot)!=NOK ALORS

		 identif=IdentMot(Cle,NB_LIBELLE,mot);
		 SI identif==ID_BREVET ALORS
		    LitEntier(ligne,&colonne,&numBrevet);
		    SI BrevetEstDansPT(abs(numBrevet))!=VRAI ALORS
		       PT[i]=abs(numBrevet);
		       i++;
		    FIN_SI
		 FIN_SI
	      FIN_SI
	   FIN_SI
	   noLigne++;
	FIN_TANTQUE

	fclose(ptFic);
     FIN_SI

     eleve++;
  FIN_TANTQUE

  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin


FONCTION int LectureDuPT(int eleve, int idBrevet)
debut
  FILE *ptFic;
  char ligne[LG_LIGNE];
  char mot[LG_LIGNE];
  int i, colonne, identif, numBrevet,noLigne=0,cr=FAUX;
  char repertoire[LG_PATH+1];

  char nomPT[LG_FIC_EXT+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  i=0;
  TANTQUE i<NB_MAX_ITEM_PT*NB_MAX_ELEVES FAIRE
     PT[i]=NON_INIT;
     i++;
  FIN_TANTQUE

  COPIER(nomPT,L_NOMFIC(eleve));
  i=0;
  TANTQUE nomPT[i]!='.' ET nomPT[i]!='\0' FAIRE
     i++;
  FIN_TANTQUE
  nomPT[i]='\0';
  strcat(nomPT,".pt");

  ptFic=fopen(nomPT,"rt");
  SI ptFic==NULL ALORS
     retour NON_INIT;
  FIN_SI

  i=0;

  TANTQUE fgets(ligne,LG_LIGNE,ptFic)!=NULL FAIRE
    colonne=0;

    SI ligne[0]!=COMMENTAIRE ALORS
      SI ChercherMot(ligne,&colonne,mot)!=NOK ALORS

        identif=IdentMot(Cle,NB_LIBELLE,mot);
	SI identif==ID_BREVET ALORS
	    LitEntier(ligne,&colonne,&numBrevet);
	    PT[i]=abs(numBrevet);
	    SI PT[i]==idBrevet ALORS
	       cr=VRAI;
	    FIN_SI
	    i++;
	FIN_SI
      FIN_SI
    FIN_SI
    noLigne++;
  FIN_TANTQUE

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour cr;

fin

/****************************/

PROCEDURE TravailAjouteDansPT(int idBrevet, int eleve, int emetteur)
debut
  int i;
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+2];
  char nomBrevet[LG_TITRE_BREVET+1];

  i=0;
  TANTQUE PT[i]!=NON_INIT ET i<NB_MAX_ITEM_PT FAIRE
     SI PT[i]==idBrevet ALORS
	Message(7,ATTRTITLE,NON_INIT,"\n  C'est dj un projet !");
	retour;
     FIN_SI
     i++;
  FIN_TANTQUE

  SI i<NB_MAX_ITEM_PT ALORS
     PT[i]=idBrevet;
  SINON
     i=0;
     TANTQUE PT[i]>0 ET i<NB_MAX_ITEM_PT FAIRE
	i++;
     FIN_TANTQUE
     SI i<NB_MAX_ITEM_PT ALORS
	PT[i]=idBrevet;
     SINON
	Message(7,ATTRTITLE,NON_INIT,"\n  Liste de projets complte !");
	retour;
     FIN_SI
  FIN_SI

  nomBrevet[0]='\0';
  comment1[0]='\0';
  comment2[0]='\0';
  LitInfosBrevet(idBrevet,nomBrevet,comment1,comment2);

  Message(13,ATTRTITLE,eleve,"\n <%s> \n\n ajout dans les projets.\n\n",nomBrevet);

  EnregFicPTEnf(eleve);

  /* on informe les dtenteurs du brevet */
  EnvoiGroupe(idBrevet, eleve, FAUX);

  /* enregistrement dans le fichier "derniers changements */
  ArchiveChangement(eleve,idBrevet,emetteur,"PT +");

fin

/****************************/

PROCEDURE TravailEnleveDuPT(int idBrevet, int eleve, int emetteur)
debut
  int i;
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+2];
  char nomBrevet[LG_TITRE_BREVET+1];
  int trouve=FAUX;

  i=0;
  TANTQUE i<NB_MAX_ITEM_PT FAIRE
     SI PT[i]==idBrevet ALORS
	PT[i]=NON_INIT;
	trouve=VRAI;
     FIN_SI
     i++;
  FIN_TANTQUE

  SI trouve==VRAI ALORS
     nomBrevet[0]='\0';
     comment1[0]='\0';
     comment2[0]='\0';
     LitInfosBrevet(idBrevet,nomBrevet,comment1,comment2);

     Message(13,ATTRTITLE,eleve,"\n <%s> \n\n a t ENLEVE de ta liste de projets.\n\n",nomBrevet);

     EnregFicPTEnf(eleve);

     /* enregistrement dans le fichier "derniers changements */
     ArchiveChangement(eleve,idBrevet,emetteur,"PT -");
  FIN_SI

fin

/****************************/

PROCEDURE TravailTermineDansPT(int idBrevet, int eleve, int emetteur)
debut
  int i,j;
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+2];
  char nomBrevet[LG_TITRE_BREVET+1];
  int trouve=FAUX;
  int PTcopie[NB_MAX_ITEM_PT];
  char titre[77+1];
  char nomCentre[LG_TITRE_BREVET+1];

  LectureDuPT(eleve,NON_INIT);
  i=0;
  TANTQUE i<NB_MAX_ITEM_PT FAIRE
     PTcopie[i]=PT[i];
     i++;
  FIN_TANTQUE

  i=0;j=0;
  TANTQUE i<NB_MAX_ITEM_PT FAIRE
     SI PTcopie[i]==idBrevet ALORS
	trouve=VRAI;
     SINON
	PT[j]=PTcopie[i];
	j++;
     FIN_SI
     i++;
  FIN_TANTQUE
  PT[j]=NON_INIT;

  SI trouve==VRAI ALORS
     nomBrevet[0]='\0';
     comment1[0]='\0';
     comment2[0]='\0';
     LitInfosBrevet(idBrevet,nomBrevet,comment1,comment2);

     Centrer(nomBrevet,nomCentre,LG_TITRE_BREVET);
     Centrer("PROJET TERMINE :",titre,LG_TITRE_BREVET);
     Message(13,ATTRTITLE,eleve,"\n%s\n\n%s",titre,nomCentre);

     EnregFicPTEnf(eleve);

     /* enregistrement dans le fichier "derniers changements */
     ArchiveChangement(eleve,idBrevet,emetteur,"PT *");
  FIN_SI

fin

/***************************/
PROCEDURE EnregFicPTEnf(int eleve)
debut
  FILE *ptFic;
  int i;
  char repertoire[LG_PATH+1];
  char nomPT[LG_FIC_EXT+1];
  char nom[LG_TITRE_BREVET+1];
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  COPIER(nomPT,L_NOMFIC(eleve));
  i=0;
  TANTQUE nomPT[i]!='.' ET nomPT[i]!='\0' FAIRE
     i++;
  FIN_TANTQUE
  nomPT[i]='\0';
  strcat(nomPT,".pt");

  ptFic=fopen(nomPT,"wt");

  i=0;
  TANTQUE i<NB_MAX_ITEM_PT FAIRE
     SI PT[i]!=NON_INIT ALORS
	nom[0]='\0';
	comment1[0]='\0';
	comment2[0]='\0';
	LitInfosBrevet(PT[i],nom,comment1,comment2);

	fprintf(ptFic,"BREVET %d %s\n",PT[i],nom);
	fprintf(ptFic,"// %s\n// %s\n",comment1,comment2);
     FIN_SI
     i++;
  FIN_TANTQUE

  fclose (ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);
fin


/***************************/

FONCTION int LectFicEleve(int idEleve)
debut
  FILE *ptFic;
  char ligne[LG_LIGNE];
  char mot[LG_LIGNE];
  int colonne=0,noLigne=0,identif,numBrevet;
  int i=1; /* ime brevet de l'lve */
  int arretLectureFichier=FAUX;

  /* ouverture du fichier eleve */
  ptFic=fopen(L_NOMFIC(idEleve),"rt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     retour NOK;
  FIN_SI

  /* Tant que (pas fin de fichier) faire */
  TANTQUE fgets(ligne,LG_LIGNE,ptFic)!=NULL ET
	  arretLectureFichier==FAUX FAIRE
    colonne=0;

    /* ignorer les commentaires */
    SI ligne[0]!=COMMENTAIRE ALORS
      /* ignorer les lignes vides, lecture du premier mot */
      SI ChercherMot(ligne,&colonne,mot)!=NOK ALORS

        /* rechercher l'identificateur du mot */
        identif=IdentMot(Cle,NB_LIBELLE,mot);
        LORSQUE (identif) VAUT
	    SOIT ID_BREVET : SI TabEleves[idEleve].NbBrevetsElev < NB_MAX_BREV_ELEV ALORS
				LitEntier(ligne,&colonne,&numBrevet);
				SI abs(numBrevet)>L_NBBREVETS() ALORS
				SINON
				   SI EnfantABrevet(idEleve,abs(numBrevet))!=1 ET
				      numBrevet>0 ALORS
				      E_BREVELEV(idEleve,i,numBrevet);
				      TabEleves[idEleve].NbBrevetsElev=TabEleves[idEleve].NbBrevetsElev+1;
				      SI numBrevet>0 ALORS
					 TabBrevets[abs(numBrevet)].NbDetenteurs=TabBrevets[abs(numBrevet)].NbDetenteurs+1;
				      FIN_SI
				      i++;
				   FIN_SI
				 FIN_SI
			     SINON
				Message(13,ATTRTITLE,idEleve,"\n\nNombre de brevets limits  %d",NB_MAX_BREV_ELEV);
				arretLectureFichier=VRAI;
			     FIN_SI
			     FIN_SOIT

        FIN_LORSQUE
      FIN_SI
    FIN_SI
    noLigne++;
  FIN_TANTQUE

  /* fermeture du fichier */
  fclose(ptFic);

  retour OK;
fin


/* ************************************** */

FONCTION int EnregFicEnf(int idEnfant)
debut
  FILE *ptFic;
  int i,idBrevet;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  ptFic=fopen(L_NOMFIC(idEnfant),"wt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour NOK;
  FIN_SI

  i=1;
  TANTQUE i<=L_NBBREVETSELEV(idEnfant) FAIRE
    idBrevet=L_BREVELEV(idEnfant,i);
    fprintf(ptFic,"BREVET %d\n",idBrevet);
    i++;
  FIN_TANTQUE

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour OK;
fin

/* ************************************** */

FONCTION int AjouBrevFicEnf(int idEnfant, int idBrevet, int AffFenetre, int emetteur)
/* AffFenetre : indicateur pour savoir si on affiche une fenetre */

debut
  int cr;
  char nomCentre[LG_TITRE_BREVET+1];
  char nom[LG_TITRE_BREVET+1];
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+1];
  char surnom[LG_SURNOM_ELEVE+1];
  char titre[77+1];
  int mois=0;
  struct date memDate;

  InitBrevets();
  InitEleves();

  cr=EnfantABrevet(idEnfant,idBrevet);

  SI cr!=1 ALORS
     getdate(&memDate);
     mois=memDate.da_mon;
     E_BREVELEV(idEnfant,L_NBBREVETSELEV(idEnfant)+1,(idBrevet+mois*1000));
     E_NBDETENTEURS(idBrevet,L_NBDETENTEURS(idBrevet)+1);
     E_NBBREVETSELEV(idEnfant,L_NBBREVETSELEV(idEnfant)+1);
     EnregFicEnf(idEnfant);

     nom[0]='\0';
     comment1[0]='\0';
     comment2[0]='\0';
     LitInfosBrevet(idBrevet,nom,comment1,comment2);
     Centrer(nom,nomCentre,LG_TITRE_BREVET);
     COPIER(surnom,L_SURNOMELEVE(idEnfant));

     /* On suppose que puisque le brevet est russi, */
     /* le travail associ au PT (s'il y tait) est termin */
     TravailTermineDansPT(idBrevet,idEnfant,emetteur);

     SI AffFenetre==VRAI ALORS
	Centrer("BREVET REUSSI :",titre,LG_TITRE_BREVET);
	Message(13,ATTRTITLE,idEnfant,"\n%s\n\n%s",titre,nomCentre);
     FIN_SI

     /* enregistrement dans le fichier derniers changements */
     ArchiveChangement(idEnfant,idBrevet,emetteur,"BL *");

  FIN_SI

  retour cr;
fin

/* ************************************** */

FONCTION int SuppBrevFicEnf(int idEnfant, int idBrevet, int AffFenetre,int emetteur)
debut
  int i,j,cr,trouve;
  char nomBrevet[LG_TITRE_BREVET+1];
  char comment1[LG_LIGNE+1], comment2[LG_LIGNE+2];
  char titre[LG_TITRE_BREVET+1];

  InitBrevets();
  InitEleves();

  E_NBBREVETSELEV(idEnfant,L_NBBREVETSELEV(idEnfant)-1);
  E_NBDETENTEURS(idBrevet,L_NBDETENTEURS(idBrevet)-1);
  trouve=FAUX;
  j=1;
  TANTQUE j<=L_NBBREVETSELEV(idEnfant) FAIRE
     i=abs(L_BREVELEV(idEnfant,j));
     SI trouve==VRAI OU i==idBrevet ALORS
	E_BREVELEV(idEnfant,j,L_BREVELEV(idEnfant,j+1));
	trouve=VRAI;
     FIN_SI
     j++;
  FIN_TANTQUE

  cr=EnregFicEnf(idEnfant);

  nomBrevet[0]='\0';
  comment1[0]='\0';
  comment2[0]='\0';
  LitInfosBrevet(idBrevet,nomBrevet,comment1,comment2);

  SI AffFenetre==VRAI ALORS
     Centrer(nomBrevet,nomBrevet,LG_TITRE_BREVET);
     Centrer("BREVET ENLEVE :",titre,LG_TITRE_BREVET);
     Message(13,(COULEUR5<<4)+COULEUR6,idEnfant,"\n%s\n\n%s",titre,nomBrevet);
  FIN_SI

  /* enregistrement dans le fichier "Derniers changements" */
  ArchiveChangement(idEnfant,idBrevet,emetteur,"BL -");

  retour cr;
fin


/* ------------------------------------------------------------------------ */
/*               LE FICHIER DE CONFIGURATION                                */
/* ------------------------------------------------------------------------ */

PROCEDURE Conv12()
debut
  int idEnfant,i,idBrevet;

  Message(13,ATTRTITLE,NON_INIT,"\n\n Conversion des fichiers des versions 1.2 et antrieures");

  idEnfant=0;
  TANTQUE idEnfant<L_NBELEVES() FAIRE
     i=0;
     TANTQUE i<L_NBBREVETSELEV(idEnfant) FAIRE
	idBrevet=L_BREVELEV(idEnfant,i);
	E_BREVELEV(idEnfant,i,abs(idBrevet)+1);
	i++;
     FIN_TANTQUE
     EnregFicEnf(idEnfant);
     idEnfant++;
  FIN_TANTQUE

  EnregConfig();
fin

/*************************************************/

PROCEDURE Conv20()
debut
  int idEnfant,i,idBrevet;

  Message(13,ATTRTITLE,NON_INIT,"\n\n Conversion des fichiers de la version 2.0");

  idEnfant=0;
  TANTQUE idEnfant<L_NBELEVES() FAIRE
     i=1;
     TANTQUE i<=L_NBBREVETSELEV(idEnfant) FAIRE
	idBrevet=L_BREVELEV(idEnfant,i);
	SI idBrevet<0 ALORS
	   E_BREVELEV(idEnfant,i,idBrevet-1);
	SINON
	   E_BREVELEV(idEnfant,i,idBrevet+1);
	FIN_SI
	i++;
     FIN_TANTQUE
     EnregFicEnf(idEnfant);
     idEnfant++;
  FIN_TANTQUE

  EnregConfig();
fin

/***********************************************************/

FONCTION int InitConfig()
debut
  FILE *PtFic;
  char repertoire[LG_PATH+1];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  /* ouverture du fichier */
  PtFic=fopen(NOM_FIC_CONFIG,"wt");
  SI PtFic==NULL ALORS
     retour FAUX;
  FIN_SI

  /* fermeture du fichier */
  fclose(PtFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour VRAI;
fin

/***********************************************************/

FONCTION int LectureConfig()
debut
  FILE *PtFic;
  char Ligne[LG_LIGNE];
  char Mot[LG_LIGNE];
  int Colonne=0,NoLigne=0,Identif, PremLigne=OUI,cr=VRAI;
  char repertoire[LG_PATH+1];
  int longueur;

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  /* ouverture du fichier */
  PtFic=fopen(NOM_FIC_CONFIG,"rt");
  SI PtFic==NULL ALORS
     retour FAUX;
  FIN_SI

  /* Tant que (pas fin de fichier) faire */
  TANTQUE fgets(Ligne,LG_LIGNE,PtFic)!=NULL FAIRE
    Colonne=0;

    /* ignorer les commentaires */
    SI Ligne[0]==COMMENTAIRE ALORS
    SINON
      /* ignorer les lignes vides, lecture du premier mot */
      SI ChercherMot(Ligne,&Colonne,Mot)==NOK ALORS
      SINON
	 /* vrification que la premire ligne contient le numro de la version du logiciel */
	 SI PremLigne==OUI ALORS
	    PremLigne=NON;
	    SI strcmp(Mot,"VERSION")==0 ALORS
	       /* recherche de la version */
	       ChercherMot(Ligne,&Colonne,Mot);
	       SI strcmp(Mot,"2.0")==0 ALORS
		  /* Version 2.0 */
		  cr=CONV20;
	       FIN_SI
	    SINON
	       /* version 1.2 ou antrieure */
	       cr=CONV12;
	    FIN_SI
	 SINON

	    /* rechercher l'identificateur du mot */
	    Identif=IdentMot(Cle,NB_LIBELLE,Mot);
	    SI Identif!=PAS_ID ALORS /* on ignore les Identifiants inconnus */
	       /* verification des parametres de l'identificateur */
	       LORSQUE (Identif) VAUT
		  SOIT ID_PASSE  : ChercherMot(Ligne,&Colonne,Mot);
				   COPIER(Mot,strupr(Mot));
				   E_PASSE(Mot);
				   FIN_SOIT

	       SOIT ID_TRONC    : LitEntier(Ligne,&Colonne,&longueur);
				  SI longueur>=2 ET longueur<=9 ALORS
				     LargeurTronc=longueur;
				  FIN_SI
				  FIN_SOIT

	       SOIT ID_USER :     MemoFinLigne(Ligne,&Colonne,Mot);
				  Mot[LG_USER]='\0';
				  SuppEspaceFin(Mot);
				  E_USER(Mot);

				   FIN_SOIT
	       SOIT ID_RESP :      MemoFinLigne(Ligne,&Colonne,Mot);
				   Mot[LG_USER]='\0';
				   SuppEspaceFin(Mot);
				   E_RESP(Mot);
				   FIN_SOIT

		   AUTRE          : /* on ignore les identifiants inconnus */
				    FIN_SOIT

	       FIN_LORSQUE
	    FIN_SI
	 FIN_SI
      FIN_SI
    FIN_SI
    NoLigne++;
  FIN_TANTQUE

  PaletteBingo();

  /* fermeture du fichier */
  fclose(PtFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour cr;

fin



PROCEDURE EnregConfig(void)
debut
  FILE *ptFic;
  char repertoire[LG_PATH+1];
  char chDate[30];

  L_REPDONNEES(repertoire);
  chdir(repertoire);
  AffichDate(chDate);

  ptFic=fopen(NOM_FIC_CONFIG,"wt");
  SI ptFic==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  fprintf(ptFic,"VERSION %s\n",VERSION);
  fprintf(ptFic,"DATE %s \n",chDate);

  SI L_MODECAT()==MODE_CATEGORIE ALORS
     fprintf(ptFic,"MODE\tCATEGORIE\n");
  FIN_SI

  fprintf(ptFic,"PASSE\t%s\n",L_PASSE());
  fprintf(ptFic,"%s\t%d\n",LIB_TRONC,LargeurTronc);
  fprintf(ptFic,"%s\t%s\n",LIB_USER,L_USER());
  fprintf(ptFic,"%s\t%s",LIB_RESP,L_RESP());

  fclose(ptFic);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  retour;
fin

/* ------------------------------------------------------------------------ */
/*               LES FICHIERS D'EXPORTATION                                 */
/* ------------------------------------------------------------------------ */

PROCEDURE EnregFicCatExport(char *nomFileExt)
debut
  char repertoire[LG_PATH+1];
  int i;
  FILE *ptFile;

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  ptFile=fopen(nomFileExt,"wt");
  SI ptFile==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  i=0;
  TANTQUE i<NB_MAX_CATEGORIE FAIRE
     fprintf(ptFile,"%s\n",Categorie[i].Libelle);
     i++;
  FIN_TANTQUE
  i=1;
  TANTQUE i<=L_NBBREVETS() FAIRE
     fprintf(ptFile,"%d\n",L_IDCATEGORIE(i));
     i++;
  FIN_TANTQUE
  fclose(ptFile);

  L_REPCOURANT(repertoire);
  chdir(repertoire);

fin

/* -------------------------------- */

PROCEDURE LectFicCatExport(char *nomFileExt)
debut
  char repertoire[LG_PATH+1];
  int i,idCat,colonne=0;
  FILE *ptFile;
  char ligne[LG_LIGNE];
  char mot[LG_LIGNE];

  L_REPDONNEES(repertoire);
  chdir(repertoire);

  ptFile=fopen(nomFileExt,"rt");
  SI ptFile==NULL ALORS
     Message(13,(COULEUR5<<4)+COULEUR10,NON_INIT,"ERREUR FICHIER");
     L_REPCOURANT(repertoire);
     chdir(repertoire);
     retour;
  FIN_SI

  i=0;
  TANTQUE fgets(ligne,LG_LIGNE,ptFile)!=NULL ET i<NB_MAX_CATEGORIE FAIRE
    /* ignorer les lignes vides, lecture du premier mot */
    colonne=0;
    MemoFinLigne(ligne,&colonne,mot);
    COPIER(Categorie[i].Libelle,mot);
    i++;
  FIN_TANTQUE

  i=1;
  REPETER
    colonne=0;
    /* ignorer les lignes vides, lecture du premier mot */
    SI LitEntier(ligne,&colonne,&idCat)!=NOK ALORS
       E_IDCATEGORIE(i,idCat);
       i++;
    FIN_SI
  JUSQUA fgets(ligne,LG_LIGNE,ptFile)==NULL OU i>L_NBBREVETS() FIN_REPETER

  fclose(ptFile);

  /* les ventuels brevets suivants rinitialiss  AUCUNE_CATEGORIE */
  TANTQUE i<=L_NBBREVETS() FAIRE
     E_IDCATEGORIE(i,AUCUNE_CATEGORIE);
     i++;
  FIN_TANTQUE

  L_REPCOURANT(repertoire);
  chdir(repertoire);

  /* Enregistrement du fichier de catgories */
  EnregFicCat();

  /* Enregistrement du fichier des brevets */
  EnregFicBrevets();

fin

/*************************************************************************/
/*                            MENU ADMINISTRATION                        */
/*************************************************************************/

/* ------------------------------------------------------------------- */

PROCEDURE far VoirMotsDePasse()
debut
  windows *W_mots;
  int idEleve;

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  W_mots=_wincreate(2,2,30,23,SIMPLE,ATTRNORMAL,ATTRBORDER,
			"Mot de passe",ATTRTITLE);
     _winselect (W_mots);

  idEleve=SelectEnfant(VRAI,NON_INIT,"");

  SI idEleve>=0 ALORS

     _winclrscr();_wingotoxy(1,1);

     _winprintf(" %s => %s      ",L_SURNOMELEVE(idEleve),L_MOTPASSE(idEleve));

     getch();


  FIN_SI

     _windispose(W_mots);


fin

PROCEDURE far MenuSuppEleves()
debut
  int idEnfant;
  /* int i;
  CHOICE *select; */
  windows *W_select;
/*  int premier;*/
  int idBrevet;
/*  char surnom[LG_SURNOM_ELEVE+3];*/

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     InitBrevets();
     InitEleves();

     W_select=_wincreate(25,3,LG_SURNOM_ELEVE+12,20,SIMPLE,ATTRNORMAL,ATTRBORDER,"SUPPRESSION D'UN ELEVE",ATTRTITLE);
     _winselect (W_select);

     REPETER

       idEnfant=SelectEnfant(FAUX,NON_INIT,"Qui part ?");


/*       premier=VRAI;i=0;
       TANTQUE i<L_NBELEVES() FAIRE
	  sprintf (surnom,"%s",L_SURNOMELEVE(i));
	  SI premier==VRAI ALORS
	     select=_choiceadd(surnom);
	     premier=FAUX;
	  SINON
	     _choiceadd(surnom);
	  FIN_SI
	  i++;
       FIN_TANTQUE
       _choiceend();

       id=_choiceread(3,2,LG_SURNOM_ELEVE+5,15,1,select);

       _choicedispose(select);*/

       SI idEnfant>=0 ALORS
	  /*Suppression d'un lve */
	  EnregFicEleves(idEnfant);

	  /* modification des ID des auteurs dans la liste des brevets */
	  idBrevet=0;
	  TANTQUE idBrevet<L_NBBREVETS() FAIRE
	     SI L_IDAUTEUR(idBrevet)==idEnfant ALORS
		E_IDAUTEUR(idBrevet,AUCUN_AUTEUR);
	     SINON
		SI L_IDAUTEUR(idBrevet)>idEnfant ET L_IDAUTEUR(idBrevet)!=AUCUN_AUTEUR ALORS
		   E_IDAUTEUR(idBrevet,L_IDAUTEUR(idBrevet)-1);
		FIN_SI
	     FIN_SI
	     idBrevet++;
	  FIN_TANTQUE
	  EnregFicBrevets();

	  Message(7,ATTRTITLE,NON_INIT,"\n   L'lve %s %s a t enlev.",
	       L_PRENOMELEVE(idEnfant),L_NOMELEVE(idEnfant));
	  InitBrevets();
	  InitEleves();
       FIN_SI

     JUSQUA idEnfant<0 FIN_REPETER

     _windispose(W_select);
  FIN_SI
fin

/* ------------------------------------------------------------------- */

PROCEDURE far MenuAjoutEleves()
debut

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     InitBrevets();
     InitEleves();

     AjoutEleves();
  FIN_SI
fin

/* -------------------------------------------------------------------- */

PROCEDURE far ModifParam()
debut
  windows *W_param;
  char passe[LG_MOT_PASSE+1];
  char nomUser[LG_USER+1];
  char nomResp[LG_USER+1];
  int cr;

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     /* choix du nom du dossier */
     W_param=_wincreate(X1_AFF_BREVETS,6,DX_AFF_BREVETS,11,
                       SIMPLE,ATTRNORMAL,ATTRBORDER,
                       "Modification des paramtres",ATTRTITLE);
     _winselect (W_param);
     _winclrscr();

     _wingotoxy(5,2);_winputs("Nom donn au responsable :");
     COPIER(nomResp,L_RESP());
     memset(nomResp+strlen(L_RESP()),' ',LG_USER-strlen(L_RESP()));
     nomResp[LG_USER]='\0';
     _getadd(5+27,2,STRING,nomResp,0,"Responsable, Instit, Philippe, Grand Schtroumpf ...");

     COPIER(passe,L_PASSE());
     memset(passe+strlen(L_PASSE()),' ',LG_MOT_PASSE-strlen(L_PASSE()));
     passe[LG_MOT_PASSE]='\0';
     _wingotoxy(5,4);_winputs("Mot de passe :");
     _getadd(5+15,4,STRING,passe,0,"Mot de passe instit ");
     _getupper();

     _wingotoxy(5,6);_winputs("Nom donn  l'utilisateur :");
     COPIER(nomUser,L_USER());
     memset(nomUser+strlen(L_USER()),' ',LG_USER-strlen(L_USER()));
     nomUser[LG_USER]='\0';
     _getadd(5+28,6,STRING,nomUser,0,"Enfant, Personne, Individu, Schtroumpf ...");

     _getshow();

     _wingotoxy(2,9);_winputs("Les noms seront pris en compte au prochain lancement de Bingo.");

     cr=_getread(NORMAL_MODE);

     SI cr!=CF4 ALORS

	E_PASSE(strupr(passe));
	E_USER(nomUser);
	E_RESP(nomResp);

	EnregConfig();
     FIN_SI

     _getdispose();
     _windispose(W_param);
     AllumeCurseurSouris();

     SI cr!=CF4 ALORS
	Message(7,ATTRTITLE,NON_INIT,"\n                  Modification russie !");
     FIN_SI

  FIN_SI

fin

/* --------------------------------------------------------------------- */

PROCEDURE far InitParam()
debut
  windows *W_param;

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     /* choix du nom du dossier */
     W_param=_wincreate(X1_AFF_BREVETS,3,DX_AFF_BREVETS,22,
                       SIMPLE,ATTRNORMAL,ATTRBORDER,
		       "Rinitialisation des paramtres",ATTRTITLE);
     _winselect (W_param);
     _winclrscr();

     InitConfig();

     _windispose(W_param);
     AllumeCurseurSouris();

     Message(13,ATTRTITLE,NON_INIT,"\n   Rinitialisation des paramtres effectue.\n\n   Quitter et relancer Bingo. ");

  FIN_SI

fin


/* --------------------------------------------------------------------- */

PROCEDURE far Ouvrir()
debut
  windows *W_ouvrir;
  char dossier[LG_DOSSIER+1];

  /* choix du nom du dossier */
  EteintCurseurSouris();
  W_ouvrir=_wincreate(X1_AFF_BREVETS,8,DX_AFF_BREVETS,4,
                       SIMPLE,ATTRNORMAL,ATTRBORDER,
		       "Ouverture d'une base de donnes",ATTRTITLE);

  _winselect (W_ouvrir);
  _winclrscr();


  COPIER (dossier,"        ");

  _wingotoxy(27,2);_winputs("Nom de la base :");
  _getadd(27+17,2,STRING,dossier,0,"Nom de la base de donnes");
  _getupper();
  _getread(NORMAL_MODE);
  _getdispose();

  EteintCurseurSouris();
  _windispose(W_ouvrir);
  AllumeCurseurSouris();

  COPIER(dossier,ArretEspace(dossier));
  SI dossier[0]!='\0' ALORS
     E_REPDONNEES(dossier);
     LORSQUE OuvertureBase() VAUT
	SOIT FICHIER_OUVERT : Message(13,ATTRTITLE,NON_INIT,"\n\n     Ouverture de la base <%s>. ",dossier);
			      FIN_SOIT
	SOIT FICHIER_ABSENT : Message(13,ATTRTITLE,NON_INIT,"\n\n     La base <%s> n'existe pas ! ",dossier);
			      FIN_SOIT
	SOIT FICHIER_ERREUR : Quitter(0);
			      FIN_SOIT
     FIN_LORSQUE
  FIN_SI
  AllumeCurseurSouris();
fin

/* -------------------------------------------------------------------- */

PROCEDURE Creation()
debut
  windows *W_creation;
/*  char nom[LG_DOSSIER+1];*/
/*  char passe[LG_MOT_PASSE+1];*/

  EteintCurseurSouris();
  /* choix du nom du dossier */
  W_creation=_wincreate(X1_AFF_BREVETS,6,DX_AFF_BREVETS,6,
                       SIMPLE,ATTRNORMAL,ATTRBORDER,
		       "Creation d'une nouvelle base de donnes",ATTRTITLE);

  _winselect (W_creation);
  _winclrscr();

  E_REPDONNEES("DONNEES");

  /* creation du dossier */

  mkdir("Donnees");
/*  Message(13,ATTRTITLE,NON_INIT,"\n     Cration russie.");*/

/*  memset(passe,' ',LG_MOT_PASSE);
  passe[LG_MOT_PASSE]='\0';
  _wingotoxy(22,4);_winputs("Mot de passe instit :");
  _getadd(22+22,4,STRING,passe,0,"Mot de passe");
  _getupper();
  _getread(NORMAL_MODE);
  _getdispose();

  COPIER(nom,ArretEspace(passe));*/

  E_PASSE(MOT_PASSE_DEFAUT);

  EnregConfig();
  E_NBBREVETS(0);
  EnregFicBrevets();

  EteintCurseurSouris();
  _windispose(W_creation);

  /* Saisie des lves */
  E_NBELEVES(0);
  AjoutEleves();

  OuvertureBase();

  AllumeCurseurSouris();

fin

/* ------------------------------------------------------------ */

PROCEDURE far ExportCat()
debut
  windows *W_exportCat;
  char nomFileExt[LG_FIC_EXT+1];
  char nomFile[LG_FIC+1];
  char titre[LG_MESSAGE+1];

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     W_exportCat=_wincreate(20,5,30,9,SIMPLE,ATTRNORMAL,ATTRBORDER,"EXPORTATION DES CATEGORIES",ATTRTITLE);
     _winselect(W_exportCat);

     _wingotoxy(8,3);_winputs("NOM DU FICHIER");
     COPIER(nomFile,"        ");
     _getadd(11,5,STRING,nomFile,0,"Entre le nom du fichier");
     _getread(NORMAL_MODE);
     _getdispose();

     COPIER(nomFile,ArretEspace(nomFile));
     COPIER(nomFileExt,nomFile);
     strcat(nomFileExt,".cat");

     _windispose(W_exportCat);

     SI nomFile[0]!='\0' ALORS
        InitBrevets();
        EnregFicCatExport(nomFileExt);
        Centrer("Exportation effectue.",titre,LG_MESSAGE);
	Message(13,ATTRTITLE,NON_INIT,"\n\n%s",titre);
        E_ACCEPTIMPORT(IMPORT_OK);
     FIN_SI
  FIN_SI

fin


/* ----------------------------------------------------------------- */

PROCEDURE far ImportCat()
debut
  windows *W_importCat;
  char nomFileExt[LG_FIC_EXT+1];
  char nomFile[LG_FIC+1];
  char titre[LG_MESSAGE+1];

/*  SI TestBD(VRAI) == NOK ALORS
     retour;
  FIN_SI*/

  SI CodeInstit()==VRAI ALORS

     W_importCat=_wincreate(20,5,30,9,SIMPLE,ATTRNORMAL,ATTRBORDER,"IMPORTATION DES CATEGORIES",ATTRTITLE);
     _winselect(W_importCat);

     _wingotoxy(8,3);_winputs("NOM DU FICHIER");
     COPIER(nomFile,"        ");
     _getadd(11,5,STRING,nomFile,0,"Entre le nom du fichier");
     _getread(NORMAL_MODE);
     _getdispose();

     COPIER(nomFile,ArretEspace(nomFile));
     COPIER(nomFileExt,nomFile);
     strcat(nomFileExt,".cat");

     _windispose(W_importCat);

     SI nomFile[0]!='\0' ALORS
        LectFicCatExport(nomFileExt);
        Centrer("Importation effectue.",titre,LG_MESSAGE);
	Message(13,ATTRTITLE,NON_INIT,"\n\n%s",titre);
     FIN_SI

  FIN_SI

fin


